/*
 *  Copyright 1994, 1995, 2000 Neil Russell.
 *  (See License)
 *  Copyright 2000, 2001 DENX Software Engineering, Wolfgang Denk, wd@denx.de
 */


#include <common.h>
#ifdef CAMEO_DISCOVERY
#include <command.h>
#include <net.h>

#include <configs/cameo/cameo_header.h>
#ifdef CAMEO_FW_UPGRADE
#include "cameo_fw.h"
#endif
#include "tftpsrv.h"
#include "bootp.h"

#undef  ET_DEBUG
/*
#define ET_DEBUG
*/

#define WELL_KNOWN_PORT 69      /* Well known TFTP port #       */
#define TIMEOUT     5       /* Seconds to timeout for a lost pkt    */
#ifndef CONFIG_NET_RETRY_COUNT
#define TIMEOUT_COUNT  3       /* # of timeouts before giving up  */
#else
#define TIMEOUT_COUNT  (CONFIG_NET_RETRY_COUNT * 2)
#endif
                    /* (for checking the image size)    */
#define HASHES_PER_LINE 65      /* Number of "loading" hashes per line  */

/*
 *  TFTP operations.
 */
#define TFTP_RRQ    1
#define TFTP_WRQ    2
#define TFTP_DATA   3
#define TFTP_ACK    4
#define TFTP_ERROR  5
#define TFTP_OACK   6


static int  TftpOurPort;        /* The UDP port at our end      */
static int  TftpClientPort;     /* The UDP port at client end       */
static int  TftpSrvTimeoutCount;
static ulong    TftpBlock;      /* packet sequence number       */
static ulong    TftpLastBlock;      /* last packet sequence number received */
static ulong    TftpBlockWrap;      /* count of sequence number wraparounds */
static ulong    TftpBlockWrapOffset;    /* memory offset due to wrapping    */
static int  TftpSrvState;

#define STATE_RRQ   1
#define STATE_DATA  2
#define STATE_TOO_LARGE 3
#define STATE_BAD_MAGIC 4
#define STATE_OACK  5
#define STATE_WRQ   6
#define STATE_ERR_ILLEGAL  7

#define TFTP_BLOCK_SIZE     512         /* default TFTP block size  */
#define TFTP_SEQUENCE_SIZE  ((ulong)(1<<16))    /* sequence number is 16 bit */

#define DEFAULT_NAME_LEN    (8 + 4 + 1)

#define TFTPSRV_TIMEOUT 10000UL /* Setup timeout to be 10 seconds. */


#ifdef CFG_DIRECT_FLASH_TFTP
extern flash_info_t flash_info[];
#endif

static __inline__ void
store_block (unsigned block, uchar * src, unsigned len)
{
    ulong offset = block * TFTP_BLOCK_SIZE + TftpBlockWrapOffset;
    ulong newsize = offset + len;
    (void)memcpy((void *)(FWUPGRADE_LOAD_MEMADDR + offset), src, len);

    if (NetBootFileXferSize < newsize)
        NetBootFileXferSize = newsize;
}

static void                     TftpSrvSend (void);
static void                     TftpSrvTimeout (void);
static char                     cmdstr[128]={0};
static tImghdr                 *pImgHeader;
static unsigned char           *pImgStart = NULL;
static unsigned char           *pImgNext = NULL;
static unsigned int             u4ImgByteCount = 0;

extern enum proto_t cmRunProtocol;


/**********************************************************************/

static void
TftpSrvSend (void)
{
    volatile uchar *    pkt;
    volatile uchar *    xp;
    int         len = 0;
    volatile ushort *s;

    /*
     *  We will always be sending some sort of packet, so
     *  cobble together the packet headers now.
     */
    pkt = NetTxPacket + NetEthHdrSize() + IP_HDR_SIZE;

    switch (TftpSrvState) {
    case STATE_RRQ:
        break;

    case STATE_DATA:
    case STATE_OACK:
        xp = pkt;
        s = (ushort *)pkt;
        *s++ = htons(TFTP_ACK);
        *s++ = htons(TftpBlock);
        pkt = (uchar *)s;
        len = pkt - xp;
        break;

    case STATE_TOO_LARGE:
        xp = pkt;
        s = (ushort *)pkt;
        *s++ = htons(TFTP_ERROR);
        *s++ = htons(3);
        pkt = (uchar *)s;
        strcpy ((char *)pkt, "File too large");
        pkt += 14 /*strlen("File too large")*/ + 1;
        len = pkt - xp;
        break;

    case STATE_BAD_MAGIC:
        xp = pkt;
        s = (ushort *)pkt;
        *s++ = htons(TFTP_ERROR);
        *s++ = htons(2);
        pkt = (uchar *)s;
        strcpy ((char *)pkt, "File has bad magic");
        pkt += 18 /*strlen("File has bad magic")*/ + 1;
        len = pkt - xp;
        break;

    case STATE_ERR_ILLEGAL:
        xp = pkt;
        s = (ushort *)pkt;
        *s++ = htons(TFTP_ERROR);
        *s++ = htons(ERR_ILLEGAL);
        pkt = (uchar *)s;
        strcpy ((char *)pkt, "Illegal firmware");
        pkt += 16 /*strlen("File has bad magic")*/ + 1;
        len = pkt - xp;
        break;
    }
    NetSendUDPPacket(NetTftpClientEther, NetTftpClientIP, TftpClientPort, TftpOurPort, len);
}


static void
TftpSrvHandler (uchar * pkt, unsigned dest, IPaddr_t sip, unsigned src, unsigned len)
{
    ushort proto;
    ushort *s;
    Ethernet_t *et;
#ifdef CAMEO_FW_UPGRADE
    tImghdr ImgHeader;
#endif

    et = (Ethernet_t *)NetRxPacket;
    if ( memcmp( ((Ethernet_t *)NetRxPacket)->et_src, NetTftpClientEther, 6) != 0 ) {
#ifdef ET_DEBUG
        printf("%s,%d  NetTftpClientEther:%02X-%02X-%02X-%02X-%02X-%02X, %02X-%02X-%02X-%02X-%02X-%02X\n",__FUNCTION__,__LINE__,
                          NetTftpClientEther[0],
                          NetTftpClientEther[1],
                          NetTftpClientEther[2],
                          NetTftpClientEther[3],
                          NetTftpClientEther[4],
                          NetTftpClientEther[5],
                          ((Ethernet_t *)NetRxPacket)->et_src[0],
                          ((Ethernet_t *)NetRxPacket)->et_src[1],
                          ((Ethernet_t *)NetRxPacket)->et_src[2],
                          ((Ethernet_t *)NetRxPacket)->et_src[3],
                          ((Ethernet_t *)NetRxPacket)->et_src[4],
                          ((Ethernet_t *)NetRxPacket)->et_src[5]
                          );
#endif
        return;
    }

    if (dest != WELL_KNOWN_PORT) {
#ifdef ET_DEBUG
        printf("%s,%d, dest:%d\n",__FUNCTION__,__LINE__,dest);
#endif
        return;
    }

    if ( TftpSrvState != STATE_WRQ && src != TftpClientPort) {
#ifdef ET_DEBUG
        printf("%s,%d,TftpSrvState:%d,  src:%d, TftpClientPort:%d\n",__FUNCTION__,__LINE__,TftpSrvState,src,TftpClientPort);
#endif
        return;
    }

    if (len < 2) {
#ifdef ET_DEBUG
        printf("%s,%d, len:%d\n",__FUNCTION__,__LINE__,len);
#endif      
        return;
    }
    len -= 2;
    /* warning: don't use increment (++) in ntohs() macros!! */
    s = (ushort *)pkt;
    proto = *s++;
    pkt = (uchar *)s;
    switch (ntohs(proto)) {

    case TFTP_WRQ:
        TftpSrvState = STATE_OACK;
        TftpBlock = 0;
        TftpClientPort = src;
        TftpSrvSend (); /* Send ACK */
        break;
    case TFTP_RRQ:
    case TFTP_ACK:
        break;
    default:
        break;

    case TFTP_OACK:
#ifdef ET_DEBUG
        printf("Got OACK: %s %s\n", pkt, pkt+strlen(pkt)+1);
#endif
        TftpSrvState = STATE_OACK;
        TftpSrvSend (); /* Send ACK */
        break;
    case TFTP_DATA:
        if (len < 2)
            return;
        len -= 2;
        TftpBlock = ntohs(*(ushort *)pkt);

        /*
         * RFC1350 specifies that the first data packet will
         * have sequence number 1. If we receive a sequence
         * number of 0 this means that there was a wrap
         * around of the (16 bit) counter.
         */
        if (TftpBlock == 0) {
            TftpBlockWrap++;
            TftpBlockWrapOffset += TFTP_BLOCK_SIZE * TFTP_SEQUENCE_SIZE;
            printf ("\n\t %lu MB reveived\n\t ", TftpBlockWrapOffset>>20);
        } else {
#ifdef ET_DEBUG
            if (((TftpBlock - 1) % 10) == 0) {
                putc ('#');
            } else if ((TftpBlock % (10 * HASHES_PER_LINE)) == 0) {
                puts ("\n\t ");
            }
#endif
        }


        if (TftpSrvState == STATE_WRQ || TftpSrvState == STATE_OACK) {
            /* first block received */
            TftpSrvState = STATE_DATA;
            TftpClientPort = src;
            TftpLastBlock = 0;
            TftpBlockWrap = 0;
            TftpBlockWrapOffset = 0;

            if (TftpBlock != 1) {   /* Assertion */
#ifdef ET_DEBUG
                printf ("\nTFTP error: "
                    "First block is not block 1 (%ld)\n"
                    "Starting again\n\n",
                    TftpBlock);
#endif
                TftpServerStart = 0;
                NetStartAgain ();
                break;
            }
#ifdef CAMEO_FW_UPGRADE
            /*  not used pointer to avoid alignment issue.
                alignment issue: if (pkt + 2) is odd mem address,
                                 data aboart exception will happen.
            */
            memcpy(&ImgHeader, pkt + 2, sizeof(tImghdr));
            ImgHeader.customer_sig = ntohs(ImgHeader.customer_sig);
            ImgHeader.board_ver = ntohl(ImgHeader.board_ver);
            ImgHeader.checksum = ntohl(ImgHeader.checksum);
            ImgHeader.flash_byte_cnt = ntohl(ImgHeader.flash_byte_cnt);
            if (!fw_check_header(&ImgHeader))
            {
                printf("Illegal FW header!\r\n");
                NetState = NETLOOP_SUCCESS;
                TftpServerStart = 0;
                TftpSrvState = STATE_ERR_ILLEGAL;
                TftpSrvSend ();

                /* Change back to CM_CDP after finishing f/w upgrading */
                cmRunProtocol = CM_CDP;
                NetLoop(cmRunProtocol);

                return;
            }
#endif
        }

        if (TftpBlock == TftpLastBlock) {
            /*
             *  Same block again; ignore it.
             */
            break;
        }

        TftpLastBlock = TftpBlock;
        NetSetTimeout (TFTPSRV_TIMEOUT, TftpSrvTimeout);

        store_block (TftpBlock - 1, pkt + 2, len);

        if (len < TFTP_BLOCK_SIZE) {
            /*
             *  We received the whole thing.  Try to
             *  run it.
             */
            NetState = NETLOOP_SUCCESS;
            TftpServerStart = 0;
            puts ("\nTFTP Transfer done\n");

#ifdef CAMEO_FW_UPGRADE
            if (!fw_check())
            {
                printf("FW checksum failed!\r\n");
                TftpSrvState = STATE_ERR_ILLEGAL;
                TftpSrvSend();
                
                /* Change back to CM_CDP after finishing f/w upgrading */
                cmRunProtocol = CM_CDP;
                NetLoop(cmRunProtocol);
                return;
            }
#endif

            TftpSrvSend ();

#ifdef CAMEO_FW_UPGRADE
            if (!fwupgrade())
            {
                printf("FW upgrade failed!\r\n");
            }

            /* Change back to CM_CDP after finishing f/w upgrading */
            cmRunProtocol = CM_CDP;
            NetLoop(cmRunProtocol);
#endif
            //printf("go to runtime\n");
            //run_command (UBOOOT_GO_RUNTIME_CMD, 0);
            return;
        }

        /*
         *  Acknoledge the block just received, which will prompt
         *  the client for the next one.
         */
        TftpSrvSend ();
        break;

    case TFTP_ERROR:
#ifdef ET_DEBUG
        printf ("\nTFTP error: '%s' (%d)\n",
                    pkt + 2, ntohs(*(ushort *)pkt));
        puts ("Starting again\n\n");
#endif
        TftpServerStart = 0;
        NetStartAgain ();
        break;
    }
}


static void
TftpSrvTimeout (void)
{

    if (++TftpSrvTimeoutCount > TIMEOUT_COUNT) {
#ifdef ET_DEBUG
        puts ("\nRetry count exceeded; starting again\n");
#endif
        TftpServerStart = 0;
        cmRunProtocol = CM_CDP;
        NetLoop(cmRunProtocol);
        /*
        NetStartAgain ();
        */
    } else {
#ifdef ET_DEBUG
        puts ("T ");
#endif
        NetSetTimeout (TFTPSRV_TIMEOUT, TftpSrvTimeout);
    }
}


void
TftpFwUpgradeStart (void)
{
    puts ("TFTP FW Upgrade Start\n");

    NetSetTimeout (TFTPSRV_TIMEOUT, TftpSrvTimeout);
    NetSetHandler(TftpSrvHandler);

    TftpServerStart = 1;
    TftpClientPort = 0;
    TftpSrvState = STATE_WRQ;
    TftpSrvTimeoutCount = 0;
    /* Use a pseudo-random port unless a specific port is set */
    TftpOurPort = WELL_KNOWN_PORT;/*1024 + (get_timer(0) % 3072);*/
    TftpBlock = 0;
}

void
TftpFwUpgradeLoop(void) {
    NetLoop(TFTPS);
}
#endif

